from django.conf import settings
import humanfriendly

# humanfriendly value
# see: https://humanfriendly.readthedocs.io/en/latest/readme.html#a-note-about-size-units  # noqa
MINIO_STORAGE_MAX_FILE_SIZE = getattr(
    settings, 'MINIO_STORAGE_MAX_FILE_SIZE', "100M")
MAX_FILE_SIZE = humanfriendly.parse_size(MINIO_STORAGE_MAX_FILE_SIZE)

assert MAX_FILE_SIZE > 0, "File size is small"

# Необходимо промигрировать базу при изменении этого значения
MINIO_STORAGE_MAX_FILE_NAME_LEN = getattr(
    settings, 'MINIO_STORAGE_MAX_FILE_NAME_LEN', 100)

assert MINIO_STORAGE_MAX_FILE_NAME_LEN > 0, "File name is small"

# Максимальное колмчество файлов в одном объекте
# Например 5 вложений в одном тикете
# None - неограничено
MINIO_STORAGE_MAX_FILES_COUNT = getattr(
    settings, 'MINIO_STORAGE_MAX_FILES_COUNT', None)

MINIO_STORAGE_USE_HTTPS = getattr(
    settings, 'MINIO_STORAGE_USE_HTTPS', False)

# Проверяем, что пользователь добавил все необходимые настройки
MINIO_STORAGE_ENDPOINT = settings.MINIO_STORAGE_ENDPOINT
MINIO_STORAGE_ACCESS_KEY = settings.MINIO_STORAGE_ACCESS_KEY
MINIO_STORAGE_SECRET_KEY = settings.MINIO_STORAGE_SECRET_KEY
MINIO_STORAGE_BUCKET_NAME = settings.MINIO_STORAGE_BUCKET_NAME

MINIO_STORAGE_CLEAN_PERIOD = getattr(
    settings, 'MINIO_STORAGE_CLEAN_PERIOD', 30)

TAGS_COUNT_MAX = 10
TAGS_CHARACTER_LIMIT = 100



# Generated by Django 2.1 on 2020-04-24 06:32

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apiqa_storage', '0003_auto_20191017_1105'),
    ]

    operations = [
        migrations.AddField(
            model_name='attachment',
            name='tags',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=100), default=list, size=None, verbose_name='Тэги'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-10-17 11:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apiqa_storage', '0002_auto_20190801_1236'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attachment',
            options={'ordering': ('-created',), 'verbose_name': 'Вложение', 'verbose_name_plural': 'Вложения'},
        ),
    ]



import uuid
from collections import namedtuple
from datetime import datetime
import mimetypes
from pathlib import PurePath
import re

from django.utils.crypto import get_random_string
from django.core.files.uploadedfile import UploadedFile
from slugify import slugify

from . import settings

RE_FILE_NAME_SLUGIFY = re.compile(r'[^\w\-.]+')
FileInfo = namedtuple(
    'FileInfo',
    'uid,  '          # uniq id of file
    'name, '          # file name
    'created, '       # datetime of create file
    'path, '          # file path in minio
    'size, '          # file size
    'content_type, '  # content type of file
    'data'            # Instance of django UploadedFile
)


def trim_name(
        name: str,
        max_length=settings.MINIO_STORAGE_MAX_FILE_NAME_LEN,
        suffix_count=2,
) -> str:
    """ Trim path name by max_length

    :param name: name with extension
    :param max_length:  max length of result name
    :param suffix_count:  max count of extension without trim
    :return: trimmed name
    """
    pure_path = PurePath(name)

    # Берем только последние suffix_count расширений
    pure_suffix = "".join(pure_path.suffixes[-suffix_count:])
    # Получаем имя без суффикса
    pure_name = name[:-len(pure_suffix)] if pure_suffix else name

    if len(pure_name) + len(pure_suffix) <= max_length:
        return name

    # Отдаем обрезанное имя с расширением
    if len(pure_suffix) <= max_length:
        return pure_name[:max_length - len(pure_suffix)] + pure_suffix

    # Если у файла длинный конец, то обрезаем его
    return pure_name[:max_length]


def slugify_name(name: str) -> str:
    """Return slugify name of file

    >>> slugify_name('test.jpg')
    test.jpg
    >>> slugify_name('тест.jpg')
    test.jpg
    >>> slugify_name('long_size_name.jpg')
    'long_size_na.jpg
    """
    slug_name = slugify(name, regex_pattern=RE_FILE_NAME_SLUGIFY)
    return slug_name


def create_path(file_name: str) -> str:
    date_now = datetime.now()
    date_path = date_now.strftime("%Y/%m/%d")
    rand_id = get_random_string(8)
    return trim_name(f"{date_path}/{rand_id}-{file_name}")


def content_type(file_name: str) -> str:
    file_type = mimetypes.guess_type(file_name, strict=False)
    return file_type[0] or "application/octet-stream"


def file_info(file: UploadedFile) -> FileInfo:
    file_name = slugify_name(file.name)

    return FileInfo(
        str(uuid.uuid4()),
        file.name,
        datetime.utcnow().isoformat(),
        create_path(file_name),
        file.size,
        content_type(file.name),
        file,
    )



# Generated by Django 2.2.2 on 2019-06-14 11:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Создано')),
                ('name', models.CharField(max_length=100, verbose_name='Имя')),
                ('path', models.CharField(max_length=512, verbose_name='Путь')),
                ('size', models.BigIntegerField(verbose_name='Размер')),
                ('bucket_name', models.CharField(max_length=255)),
                ('content_type', models.CharField(max_length=255)),
                ('object_id', models.CharField(blank=True, max_length=255, null=True)),
                ('object_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Вложение',
                'verbose_name_plural': 'Вложения',
            },
        ),
    ]



from datetime import datetime, timedelta

from celery import shared_task

from apiqa_storage.models import Attachment
from apiqa_storage import settings


@shared_task()
def purge_attachments():
    """ Удаляет непривязанные вложения
    старше MINIO_STORAGE_CLEAN_PERIOD дней
    """
    now = datetime.now()
    delta = timedelta(days=settings.MINIO_STORAGE_CLEAN_PERIOD)

    Attachment.objects.filter(
        created__lt=now - delta,
        object_id__isnull=True,
    ).delete()



from . import settings
from rest_framework.exceptions import ValidationError


def file_size_validator(attach_file):
    if attach_file.size > settings.MAX_FILE_SIZE:
        raise ValidationError(
            f'Max size of attach file: '
            f'{settings.MINIO_STORAGE_MAX_FILE_SIZE}'
        )



import uuid

from django.conf import settings as django_settings
from django.contrib.contenttypes.fields import (
    GenericForeignKey, GenericRelation
)
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.utils.translation import gettext as _

from . import settings
from .managers import AttachmentQuerySet


class Attachment(models.Model):
    uid = models.UUIDField(
        primary_key=True,
        default=uuid.uuid4,
        editable=False
    )
    created = models.DateTimeField(
        verbose_name=_('Создано'),
        editable=False,
        auto_now_add=True
    )
    name = models.CharField(
        verbose_name=_('Имя'),
        max_length=255
    )
    path = models.CharField(
        verbose_name=_('Путь'),
        max_length=512
    )
    size = models.BigIntegerField(
        verbose_name=_('Размер')
    )
    bucket_name = models.CharField(
        max_length=255
    )
    content_type = models.CharField(
        max_length=255
    )
    user = models.ForeignKey(
        to=django_settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL
    )
    object_content_type = models.ForeignKey(
        to=ContentType,
        null=True,
        blank=True,
        on_delete=models.CASCADE
    )
    object_id = models.CharField(
        max_length=255,
        null=True,
        blank=True
    )
    content_object = GenericForeignKey(
        ct_field='object_content_type',
        fk_field='object_id'
    )
    tags = ArrayField(
        base_field=models.CharField(
            max_length=settings.TAGS_CHARACTER_LIMIT),
        verbose_name=_('Тэги'),
        default=list,
    )

    objects = AttachmentQuerySet.as_manager()

    class Meta:
        verbose_name = _('Вложение')
        verbose_name_plural = _('Вложения')
        ordering = ('-created',)

    def __str__(self):
        return self.path

    def delete(self, *args, **kwargs):
        from apiqa_storage.serializers import delete_file
        if not self.content_object:
            delete_file(self)
            return super().delete(*args, **kwargs)


class ModelWithAttachmentsMixin(models.Model):
    attachments = GenericRelation(
        to=Attachment,
        object_id_field='object_id',
        content_type_field='object_content_type'
    )

    class Meta:
        abstract = True



from urllib.parse import quote

from django.core.files import File
from django.http import FileResponse
from rest_framework.generics import get_object_or_404
from rest_framework.views import APIView

from .http.range import parse_http_range, http_range_valid
from .http.response import PartialHttpResponse, HttpResponseNotSatisfiable
from .minio_storage import storage
from .models import Attachment


class AttachmentView(APIView):
    def get_queryset(self):
        return Attachment.objects.all()

    def get(self, request, *args, **kwargs):
        user_filter = {'user': request.user} if kwargs.get('from_user') else {}
        queryset = self.get_queryset()
        attachment = get_object_or_404(
            queryset, uid=kwargs['attachment_uid'], **user_filter
        )

        http_range = request.META.get('HTTP_RANGE')
        if http_range:
            ranges = parse_http_range(http_range, attachment.size)
            if not http_range_valid(ranges):
                return HttpResponseNotSatisfiable(attachment.size)

            contents = []
            for file_range in ranges:
                minio_file_resp = storage.file_partial_get(
                    attachment.path, attachment.bucket_name,
                    offset=file_range.start, length=len(file_range)
                )
                contents.append(minio_file_resp)

            resp = PartialHttpResponse(
                ranges, contents, content_type=attachment.content_type
            )

        else:
            minio_file_resp = storage.file_get(
                attachment.path, attachment.bucket_name
            )

            resp = FileResponse(
                File(name=attachment.name, file=minio_file_resp),
                filename=attachment.name
            )
            resp['Content-Length'] = attachment.size
            resp['Content-Disposition'] = "filename*=utf-8''{}".format(quote(
                attachment.name))

        resp["Accept-Ranges"] = "bytes"
        return resp



from typing import Optional, List


class Range:
    def __init__(self, start: int, finish: int, size: int):
        self.start = start
        self.finish = finish
        self.size = size

    @property
    def valid(self) -> bool:
        return 0 <= self.start <= self.finish < self.size

    def __len__(self) -> int:
        return self.finish - self.start + 1

    def __str__(self) -> str:
        return f'bytes {self.start}-{self.finish}/{self.size}'

    def __repr__(self) -> str:
        return f'Range({self.start}, {self.finish}, {self.size})'

    def __eq__(self, other) -> bool:
        return self.start == other.start and self.finish == other.finish \
               and self.size == self.size


def parse_http_range(range_header: Optional[str], size: int) -> \
        Optional[List[Range]]:
    """
    Parse a byte range header as specified by HTTP RFC2616 section 14.35.1.
    Takes the byte range header as well as the size of the file requested and
    returns a list of tuples (start, finish) for the byte range specified.
    """
    if range_header is None or '=' not in range_header:
        return None

    _, byte_ranges = range_header.split('=', 1)

    ranges = byte_ranges.split(',')

    results = []
    for byte_range in ranges:
        byte_range = byte_range.strip()

        if '-' not in byte_range:
            return None

        if byte_range.startswith('-'):
            # Get the last x bytes of the file
            try:
                start = size + int(byte_range)
                if start < 0:
                    return None
            except ValueError:
                return None

            finish = size - 1
        else:
            start, finish = byte_range.split('-', 1)
            try:
                start = int(start)
            except ValueError:
                return None

            if finish:
                try:
                    finish = int(finish)
                except ValueError:
                    return None

                if finish > size:
                    finish = size - 1
            else:
                finish = size - 1

            if start > finish:
                return None

        results.append(Range(start, finish, size))

    return results


def http_range_valid(ranges: Optional[List[Range]]) -> bool:
    if not ranges:
        return False

    return all([r.valid for r in ranges])



from django.http import StreamingHttpResponse, HttpResponse
from typing import List, BinaryIO, Mapping

from apiqa_storage.http.range import Range


class PartialHttpResponse(StreamingHttpResponse):
    status_code = 206
    separator = "THIS_STRING_SEPARATES"

    def __init__(self, ranges: List[Range], contents: List[BinaryIO],
                 *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ranges = ranges
        self._contents = contents

        if len(self._contents) > 1:
            # We have a multipart/byteranges response.
            self.content_type = self['Content-Type']
            self['Content-Type'] = f"multipart/byteranges; " \
                                   f"boundary={self.separator}"
        else:
            self['Content-Length'] = len(ranges[0])
            self['Content-Range'] = str(ranges[0])

    def __iter__(self):
        for content, byte_range in zip(self._contents, self.ranges):
            data = content.read()
            if len(self._contents) > 1:
                yield self.make_bytes(f"--{self.separator}\r\n")
                headers = {
                    'Content-Type': self.content_type,
                    'Content-Range': str(byte_range)
                }
                yield self.serialize_headers(headers) + b'\r\n\r\n'
                yield self.make_bytes(data) + b'\r\n'
            else:
                yield self.make_bytes(data)
        if len(self._contents) > 1:
            yield self.make_bytes(f"--{self.separator}--")

    def serialize_headers(self, headers: Mapping[str, str] = None) -> bytes:
        """HTTP headers as a bytestring."""
        if headers is None:
            headers = self._headers

        def to_bytes(val, encoding):
            return val if isinstance(val, bytes) else val.encode(encoding)

        output = [
            (b': '.join([to_bytes(key, 'ascii'), to_bytes(value, 'latin-1')]))
            for key, value in headers.items()
        ]
        return b'\r\n'.join(output)


class HttpResponseNotSatisfiable(HttpResponse):
    status_code = 416

    def __init__(self, length: int, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self['Content-Range'] = f'bytes */{length}'



from django.db import models


class AttachmentQuerySet(models.QuerySet):
    def delete(self, *args, **kwargs):
        for obj in self:
            obj.delete()
        super().delete(*args, **kwargs)



from django.urls import path

from .routers import router
from .views import AttachmentView

urlpatterns = [  # noqa: pylint=invalid-name
    path('<uuid:attachment_uid>', AttachmentView.as_view(),
         name='user-attachments', kwargs={'from_user': True})
] + router.urls



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from .models import Attachment


@admin.register(Attachment)
class AttachmentAdmin(admin.ModelAdmin):
    list_display = (
        'uid', 'created', '_name', 'size', 'content_type',
        'object_content_type', 'object_id'
    )
    search_fields = (
        'uid', 'name', 'content_type', 'object_id', 'bucket_name'
    )
    list_filter = ('content_type', 'bucket_name')
    readonly_fields = ('_name',)

    def _name(self, obj):
        url = reverse('staff-attachments', kwargs={'attachment_uid': obj.uid})
        return format_html(f'<a href="{url}">{obj.name}</a>')
    _name.short_description = _('Имя')
    _name.admin_order_field = 'name'

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = queryset.select_related('user', 'object_content_type')
        return queryset



# Generated by Django 2.2.3 on 2019-08-01 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apiqa_storage', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attachment',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Имя'),
        ),
    ]



import minio
from urllib3 import HTTPResponse

from . import settings
from .files import FileInfo


def create_minio_client():
    client = minio.Minio(
        endpoint=settings.MINIO_STORAGE_ENDPOINT,
        access_key=settings.MINIO_STORAGE_ACCESS_KEY,
        secret_key=settings.MINIO_STORAGE_SECRET_KEY,
        secure=settings.MINIO_STORAGE_USE_HTTPS,
    )
    return client


class Storage:
    def __init__(self, bucket_name: str = None):
        if bucket_name is not None:
            self.bucket_name = bucket_name
        else:
            self.bucket_name = settings.MINIO_STORAGE_BUCKET_NAME
        self.client = create_minio_client()

    def get_bucket_name(self, bucket_name: str) -> str:
        return bucket_name if bucket_name is not None else self.bucket_name

    def file_get(self, name: str, bucket_name: str = None) -> HTTPResponse:
        return self.client.get_object(
            bucket_name=self.get_bucket_name(bucket_name),
            object_name=name,
        )

    def file_partial_get(self, name: str, bucket_name: str = None,
                         offset: int = 0, length: int = 0) -> HTTPResponse:
        return self.client.get_partial_object(
            bucket_name=self.get_bucket_name(bucket_name),
            object_name=name,
            offset=offset,
            length=length,
        )

    def file_put(self, file_info: FileInfo) -> str:
        return self.client.put_object(
            bucket_name=self.bucket_name,
            object_name=file_info.path,
            data=file_info.data,
            length=file_info.size,
            content_type=file_info.content_type,
        )

    def file_delete(self, name: str):
        self.client.remove_object(self.bucket_name, name)


storage = Storage()  # noqa



import logging
import uuid
from typing import Union

from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from . import settings
from .files import FileInfo, file_info
from .minio_storage import storage
from .models import Attachment
from .validators import file_size_validator

logger = logging.getLogger('apiqa-storage')  # noqa


__all__ = [
    'AttachmentSerializer',
    'AttachmentsSerializerMixin'
]


def delete_file(attach_file_info: Union[FileInfo, Attachment]):
    # noinspection PyBroadException
    try:
        storage.file_delete(attach_file_info.path)
    except Exception:  # noqa
        logger.exception("Delete file failed: %s from bucket: %s",
                         attach_file_info.path, storage.bucket_name)


class AttachmentSerializer(serializers.ModelSerializer):
    file = serializers.FileField(write_only=True, required=True,
                                 validators=[file_size_validator])
    tags = serializers.ListField(
        child=serializers.CharField(
            max_length=settings.TAGS_CHARACTER_LIMIT),
        required=False, max_length=settings.TAGS_COUNT_MAX)

    class Meta:
        model = Attachment
        fields = (
            'file', 'uid', 'created', 'name', 'size', 'content_type', 'tags'
        )
        read_only_fields = (
            'uid', 'created', 'name', 'size', 'content_type', 'tags'
        )

    def create(self, validated_data):
        user = self.context['request'].user
        custom_uid = self.context['request'].query_params.get('uid')
        if custom_uid:
            try:
                uuid.UUID(custom_uid)
            except ValueError:
                raise ValidationError("Incorrect uid")
            if Attachment.objects.filter(uid=custom_uid).exists():
                raise ValidationError(
                    f'Attachment with uid = {custom_uid} already exists.'
                )
        attach_file = validated_data.pop('file')
        attach_file_info = file_info(attach_file)
        storage.file_put(attach_file_info)
        data = {
            'uid': custom_uid or attach_file_info.uid,
            'bucket_name': storage.bucket_name,
            'name': attach_file_info.name,
            'path': attach_file_info.path,
            'size': attach_file_info.size,
            'content_type': attach_file_info.content_type,
            'user': user
        }
        validated_data.update(data)
        try:
            return super().create(validated_data)
        except Exception:
            # Delete files if save model failed
            delete_file(attach_file_info)
            raise


class AttachmentsSerializerMixin(serializers.Serializer):
    attachments = AttachmentSerializer(many=True, read_only=True)
    attachment_ids = serializers.PrimaryKeyRelatedField(
        many=True, write_only=True, queryset=Attachment.objects.all(),
        source='attachments', required=False
    )

    def create(self, validated_data):
        attachments = validated_data.pop('attachments', [])
        instance = super().create(validated_data)
        instance.attachments.set(attachments)
        return instance

    @staticmethod
    def validate_attachment_ids(value):
        if len(value) > settings.MINIO_STORAGE_MAX_FILES_COUNT:
            raise serializers.ValidationError(
                _('Max files count: %s' % settings
                  .MINIO_STORAGE_MAX_FILES_COUNT))
        return value



# To use a consistent encoding
from codecs import open as codec_open
from os import path
# Always prefer setuptools over distutils
from setuptools import setup, find_packages

HERE_PATH = path.abspath(path.dirname(__file__))

with codec_open(path.join(HERE_PATH, 'README.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

with codec_open(path.join(HERE_PATH, 'requirements.txt'),
                encoding='utf-8') as f:
    REQUIREMENTS = [
        line for line in f.readlines()
        if line and not line.startswith('#')
    ]

with codec_open(path.join(HERE_PATH, 'requirements.dev.txt'),
                encoding='utf-8') as f:
    REQUIREMENTS_DEV = [
        line for line in f.readlines()
        if line and not line.startswith('#')
    ]

setup(
    name='apiqa-storage',
    version='2.7',
    description='Apiqa user storage backend for django projects',
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/pik-software/apiqa-storage.git',
    author='pik-software',
    author_email='no-reply@pik-software.ru',
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='apiqo django',

    packages=find_packages(exclude=[
        'contrib', 'docs', 'tests', 'test_project',
        'tests_storage', 'tests_storage.*',
    ]),

    # https://packaging.python.org/en/latest/requirements.html
    install_requires=REQUIREMENTS,
    python_requires='~=3.6',

    # List additional groups of dependencies here
    #   $ pip install sampleproject[dev]
    extras_require={
        'dev': REQUIREMENTS_DEV,
    },

    project_urls={
        'Bug Reports': 'https://gitlab.pik-software.ru/'
                       'apiqa/apiqa-storage/issues',
        'Funding': 'https://gitlab.pik-software.ru/apiqa/apiqa-storage',
        'Say Thanks!': 'https://saythanks.io/to/pik_software',
        'Source': 'https://gitlab.pik-software.ru/apiqa/apiqa-storage',
    },
)



from rest_framework import viewsets, mixins
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAuthenticated
from django.utils.translation import ugettext_lazy as _

from .models import Attachment
from .serializers import AttachmentSerializer


class AttachmentViewSet(viewsets.GenericViewSet,
                        mixins.CreateModelMixin,
                        mixins.DestroyModelMixin):
    """
    create:
        Upload file
    ---
    destroy:
        Delete file
    """
    permission_classes = (IsAuthenticated,)
    serializer_class = AttachmentSerializer
    queryset = Attachment.objects.all()

    def perform_destroy(self, instance):
        if instance.content_object:
            raise ValidationError(
                _("Delete attachments with relations not allowed"))
        super().perform_destroy(instance)



from django.urls import path

from .routers import router
from .views import AttachmentView

urlpatterns = [
    path('<uuid:attachment_uid>', AttachmentView.as_view(),
         name='staff-attachments')
] + router.urls
